/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Fence;
import cz.xelfi.quoridor.Player;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

/** Convertor of a board to Image.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
final class BoardImage  {
    static Image draw(Board b, int fieldSize) {
        int fifth = fieldSize / 10;

        BufferedImage img = new BufferedImage(fieldSize * 9, fieldSize * 9, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) img.getGraphics();

        g.setColor(Color.lightGray);
        for (int i = 0; i < 9; i++) {
            for (int j = 0; j < 9; j++) {
                final Rectangle r = new Rectangle(i * fieldSize, j * fieldSize, fieldSize, fieldSize);
                g.fillRect(r.x + fifth, r.y + fifth, r.width - fifth * 2, r.height - fifth * 2);
            }
        }


        g.setColor(Color.BLACK);
        for (Fence f : b.getFences()) {
            int w, h;
            switch (f.getOrientation()) {
                case HORIZONTAL: w = fieldSize - fifth; h = fifth; break;
                case VERTICAL: w = fifth; h = fieldSize - fifth; break;
                default: throw new IllegalStateException();
            }
            int column = (f.getColumn() - 'A') + 1;
            int row = 9 - f.getRow();
            Rectangle r = new Rectangle(
                column * fieldSize - w,
                row * fieldSize - h,
                2 * w,
                2 * h
            );
            g.fill(r);
        }

        int cnt = 0;
        Color[] colors = { Color.WHITE, Color.BLACK, Color.ORANGE, Color.MAGENTA };
        int diametr = fieldSize - fifth * 4;
        for (Player p : b.getPlayers()) {
            int column = p.getColumn();
            int row = 8 - p.getRow();
            final Rectangle r = new Rectangle(
                column * fieldSize + 2 * fifth,
                row * fieldSize + 2 * fifth,
                diametr,
                diametr
            );
            g.setColor(colors[cnt]);
            g.fillOval(r.x, r.y, r.width, r.height);
            cnt++;
        }

        g.setColor(Color.BLACK);
        final Font f = new Font("Monospace", Font.BOLD, fifth * 2);
        g.setFont(f);
        for (int i = 0; i < 8; i++) {
            final char ch = (char) ('A' + i);
            g.drawString(Character.toString(ch), i * fieldSize + fieldSize - fifth, fifth + f.getSize() - f.getBaselineFor(ch));
        }
        for (int i = 0; i < 8; i++) {
            String s = "" + (8 - i);
            g.drawString(s, fifth, i * fieldSize + fieldSize - fifth + f.getSize() - f.getBaselineFor(s.charAt(0)));
        }

        return img;
    }
}
